// =======================================================================
//                                 BFG-9500
//
// Original code by Jonathan E. Wright
// Model by Pavel Grodek
// Modified for the Ultimate Weapons Mod by Andy Brennan
// =======================================================================

// spawns an energy ball at the entity's position and applies the damage
void (entity victim) bfg_spawntracehit =
{
	local   entity  bfg_hit;

	bfg_hit = spawn ();
	bfg_hit.enemy = victim;
	bfg_hit.solid = SOLID_NOT;
	bfg_hit.movetype = MOVETYPE_NONE;
	bfg_hit.origin = bfg_hit.enemy.origin;

	 setmodel (bfg_hit, "progs/bfg.mdl");
	bfg_hit.frame = 2;
	setsize (bfg_hit, '2 2 2', '2 2 2');
	setorigin (bfg_hit, victim.origin);

	bfg_hit.nextthink = time + 0.1;
	bfg_hit.think = bfg_tracehit1;

	bfg_hit.classname = "bfg_tracehit";
	bfg_hit.targetname = victim.classname;
};

// spawns an energy ball at the entity's position and applies the damage
void() bfg_applymultidamage =
{
	if (!multi_ent)
		return;

	T_Damage (multi_ent, self, self, multi_damage);

	bfg_spawntracehit (multi_ent);
};

void(entity hit, float damage) bfg_addmultidamage =
{
	if (!hit)
		return;

	if (hit != multi_ent)
	{
		bfg_applymultidamage ();
		multi_damage = damage;
		multi_ent = hit;
	}
	else
		multi_damage = multi_damage + damage;
};

void () bfg_areadamage =
{
	local vector  org, end, dir;
	local float   trace_count, j;

	// a cone of 75 rays is now fired in the direction of the shot, 
	// but from the player's current position

	makevectors (self.v_angle);

	// make trace begin about 30 degrees left of the direction fired
	org = self.owner.origin;
	end = org + (v_forward * 1200) - (v_right * 950);

	ClearMultiDamage ();

	trace_count = 1;
	j = -2;  // -1 because it's used to multiply the v_up vector and 
		 // tilt the trace above and below the firing plane

	// original bfg had 20 traces, but they seem to have been more 
	// substantial, so I made more, in four rows one above and one 
	// below the firing plane  

	while (j <= 2)
	{
		trace_count = 1;
		org = self.owner.origin;
		end = org + (v_forward * 1200) - (v_right * 950) + 
		      (v_up * (j * 125));

		while (trace_count <= 40)
		 {
			end = end + (v_right * 45);

			traceline (org, end, FALSE, self);
			if (trace_ent.takedamage)
			{
				SpawnBlood (org, '0 0 0', 20);
	// original BFG traces did from 5 to 15 points of damage
	// increased here to give the same apparent affect against grunts 
				bfg_addmultidamage (trace_ent, 7 + random () * 10);
			}
      // add three degrees to get the direction for the next trace

			trace_count = trace_count + 1;
		}
		j = j + 1;
	}

	bfg_applymultidamage ();
};

void () bfg_touch =
{
	local float rand;
	
	// hit something that bleeds
	if (other.takedamage)
	{
		// direct hit on an entity
		spawn_touchblood (9);
    // original BFG does 100 - 700 points of damage, but nothing in Quake  
    // has this much health, so the damage has been changed to 70-300 points
		T_Damage (other, self, self.owner, 70 + 230 * random ());
	}

	setmodel (self, "progs/s_bfg.spr");
	self.frame = 2;
	self.velocity = '0 0 0';
	self.nextthink = time + 0.1;
	self.think = bfg_impact1;
	self.touch = SUB_Null;
};

void () W_FireBFG =
{
	local vector  dir;
	local entity  energyball;

	makevectors (self.v_angle);

	if (self.ammo_cells < 20)
	{
		self.dest_y = self.weapon;
		self.weapon = W_BestWeapon ();
		W_SetCurrentAmmo ();
		return;
	}

	self.ammo_cells = self.ammo_cells - 20;
	self.currentammo = floor(self.ammo_cells / 20);
	self.attack_finished = time + 2;

	dir = normalize ((self.origin + (v_forward * 1000)) - self.origin);

	// launch the energy ball
	energyball = spawn ();
	energyball.owner = self;
	energyball.movetype = MOVETYPE_FLYMISSILE;
	energyball.solid = SOLID_BBOX;

	energyball.touch = bfg_touch;
	energyball.classname = "bfg_energyball";
	energyball.think = SUB_Remove;
	energyball.nextthink = time + 6;
	setmodel (energyball, "progs/bfg.mdl");
	energyball.frame = 1;
	setsize (energyball, '-0 -0 -0', '0 0 0');
	setorigin (energyball, self.origin + '0 0 16');

	energyball.velocity = dir * 500;
	energyball.angles = vectoangles (dir);

	energyball.v_angle = self.v_angle;

	self.punchangle_x = -2;
	self.attack_finished = time + 1.5;
};

